/*****************************************************************
* IBM Confidential OCO Source Material
* 5724-I63, 5724-H88, 5655-N01, 5733-W60 (C) COPYRIGHT International Business Machines Corp. 1997, 2005
* The source code for this program is not published or otherwise divested
* of its trade secrets, irrespective of what has been deposited with the
* U. S. Copyright Office.
******************************************************************/
package com.ibm.ws.install.factory.base.aspects.logging;

import com.ibm.ws.install.factory.base.util.logging.InstallFactoryLogger;
import java.util.logging.Level;

public abstract aspect InstallFactoryTrace {
    
    public abstract pointcut allMethodCalls ();

    /** 
     * Exclude InstallFactoryTrace aspects from exception reporting to avoid unwanted
     * recursion
     */
    private pointcut excluded () : within(InstallFactoryTrace+);

    /** 
     */
    private pointcut noneTraceableSubsystemMethodCalls():
		execution( * com.ibm.ws.install.factory.base.util.logging..*.* (..) ) ||
		execution( * com.ibm.ws.install.factory.base.xml..*.* (..) ) ||
		execution( * com.ibm.ws.install.factory.was.xml..*.* (..) );
    
    /** 
     */
    private pointcut loggableMethodCalls():		
		allMethodCalls() && 
		!noneTraceableSubsystemMethodCalls() &&
		!excluded();

	/**
	 * Pointcut that defines all join points that catch exceptions in the
	 * install factory.
	 * <p>
	 * @param e	The exception that has been caught. 
	 */
    private pointcut caughtException( Throwable e ):
		args( e ) &&
		handler( Throwable+ ) ;

	/**
	 * Pointcut that defines all loggable exceptions.
	 */
    private pointcut loggableExceptions( Throwable e ):
		caughtException( e ) && 
		!noneTraceableSubsystemMethodCalls();
	
	/**
	 * Runs at the entry point of any loggable method.
	 */
	before(): loggableMethodCalls() {
		InstallFactoryLogger.traceMessage(Level.FINER, thisJoinPointStaticPart.getSourceLocation().getWithinType().toString(),
			thisJoinPointStaticPart.getSignature().getName(), 
			"Entering: " + thisJoinPointStaticPart.getSignature().toString() );
		
		Object[] aoArgs = thisJoinPoint.getArgs();
		String sArgs = new String();
		for ( int i = 0; i < aoArgs.length; i++ ) {
			String sArgThis = null;
			if ( aoArgs[ i ] != null ) 	{
				sArgThis = aoArgs[ i ].toString();
			}
			else {
				sArgThis = "null";
			}
			
			if ( i < ( aoArgs.length - 1 ) ) {
				sArgs = sArgs + sArgThis + ", ";
			}
			else {
				sArgs = sArgs + sArgThis;
			}
		}
		
		InstallFactoryLogger.traceMessage( Level.FINEST,
			thisJoinPointStaticPart.getSourceLocation().getWithinType().toString(),
			thisJoinPointStaticPart.getSignature().getName(),
			"Arguments: " + sArgs );
	}

	/**
	 * Runs at the exit point of any loggable method.
	 */
	after(): loggableMethodCalls() { 
		InstallFactoryLogger.traceMessage( Level.FINER, thisJoinPointStaticPart.getSourceLocation().getWithinType().toString(),
			thisJoinPointStaticPart.getSignature().getName(), 
			"Exiting: " + thisJoinPointStaticPart.getSignature().toString() );
	}

	/**
	 * Runs whenever a loggable exception is caught, before the exception 
	 * handler runs.
	 * <p>
	 * @param e	The exception that has been caught.
	 */

	before( Throwable e ): loggableExceptions( e ) 	{
		InstallFactoryLogger.traceException( Level.WARNING,
			thisJoinPointStaticPart.getSourceLocation().getWithinType().toString(),
			thisJoinPointStaticPart.getSignature().getName(),
			"Exception: " + e.getMessage(),	e );
	}
}
